MODULE DrakonComponents; (** AUTHOR "SAGE"; PURPOSE "DRAKON Componets"; *)

IMPORT
	XML, WMComponents, WMProperties, Strings;
	
CONST
	Ok* = 0;
	Error* = -1;
	
TYPE

	Base = WMComponents.Component;
	
	(* DrakonDocument *)
	DrakonDocument* = OBJECT(Base)

	PROCEDURE &Init;
	BEGIN
		Init^;
		SetNameAsString(strDrakonDocument)
	END Init;
	
	PROCEDURE AddContent*(c: XML.Content);
	BEGIN
		ASSERT(c IS DrakonSchema);
		AddContent^(c);
		c(DrakonSchema).document := SELF;
	END AddContent;
	
	END DrakonDocument;

	(* DrakonSchema *)
	DrakonSchema* = OBJECT(Base)
	VAR
		document: DrakonDocument;

	PROCEDURE &Init;
	BEGIN
		Init^;
		SetNameAsString(strDrakonSchema)
	END Init;
	
	PROCEDURE AddContent*(c: XML.Content);
	BEGIN
		ASSERT(c IS DrakonElement);
		c(DrakonElement).elIndex.Set(GetNumberOfContents());
		AddContent^(c);
		c(DrakonElement).schema := SELF;
	END AddContent;
	
	END DrakonSchema;
	
	(* Base DrakonElement *)
	DrakonElement* = OBJECT(Base)
	VAR
		schema-: DrakonSchema;
		elLeft-, elTop-, elRight-, elBottom-, elExtra-: DrakonElement;
		elIndex, elIndexLeft, elIndexTop, elIndexRight, elIndexBottom, elIndexExtra-: WMProperties.Int32Property;
		code-, text-: WMProperties.StringProperty;
		
	PROCEDURE &Init*;
	BEGIN
		Init^;
		NEW(elIndex, PrototypeIndex, NIL, NIL); properties.Add(elIndex);
		NEW(elIndexLeft, PrototypeIndexL, NIL, NIL); properties.Add(elIndexLeft);
		NEW(elIndexTop, PrototypeIndexT, NIL, NIL); properties.Add(elIndexTop);
		NEW(elIndexRight, PrototypeIndexR, NIL, NIL); properties.Add(elIndexRight);
		NEW(elIndexBottom, PrototypeIndexB, NIL, NIL); properties.Add(elIndexBottom);
		NEW(elIndexExtra, PrototypeIndexX, NIL, NIL); properties.Add(elIndexExtra);
		NEW(code, PrototypeCode, NIL, NIL); properties.Add(code);
		NEW(text, PrototypeText, NIL, NIL); properties.Add(text);
		elIndex.Set(-1);
		elIndexLeft.Set(-1);
		elIndexTop.Set(-1);
		elIndexRight.Set(-1);
		elIndexBottom.Set(-1);
		elIndexExtra.Set(-1);
	END Init;
	
	PROCEDURE LinkTopElement*(element: DrakonElement);
	BEGIN
		ASSERT(SELF.schema = element.schema);
		elTop := element;
		elIndexTop.Set(element.elIndex.Get());
		IF element.elBottom # SELF THEN
			element.LinkBottomElement(SELF)
		END
	END LinkTopElement;
	
	PROCEDURE LinkBottomElement*(element: DrakonElement);
	BEGIN
		ASSERT(SELF.schema = element.schema);
		elBottom := element;
		elIndexBottom.Set(element.elIndex.Get());
		IF element.elTop # SELF THEN
			element.LinkTopElement(SELF)
		END
	END LinkBottomElement;
	
	PROCEDURE LinkLeftElement*(element: DrakonElement);
	BEGIN
		ASSERT(SELF.schema = element.schema);
		elLeft := element;
		elIndexLeft.Set(element.elIndex.Get());
		IF element.elRight # SELF THEN
			element.LinkRightElement(SELF)
		END
	END LinkLeftElement;
	
	PROCEDURE LinkRightElement*(element: DrakonElement);
	BEGIN
		ASSERT(SELF.schema = element.schema);
		elRight := element;
		elIndexRight.Set(element.elIndex.Get());
		IF element.elLeft # SELF THEN
			element.LinkLeftElement(SELF)
		END
	END LinkRightElement;

	END DrakonElement;

	(* DrakonElementHead *)
	DrakonElementHead* = OBJECT(DrakonElement)

	PROCEDURE &Init*;
	BEGIN
		Init^;
		SetNameAsString(strDrakonElementHead)
	END Init;

	END DrakonElementHead;

	(* DrakonElementAction *)
	DrakonElementAction* = OBJECT(DrakonElement)

	PROCEDURE &Init*;
	BEGIN
		Init^;
		SetNameAsString(strDrakonElementAction)
	END Init;

	END DrakonElementAction;

	(* DrakonElementEnd *)
	DrakonElementEnd* = OBJECT(DrakonElement)

	PROCEDURE &Init*;
	BEGIN
		Init^;
		text.Set(strEnd);
		SetNameAsString(strDrakonElementEnd)
	END Init;

	END DrakonElementEnd;
	
	(* DrakonElementNode *)
	DrakonElementNode* = OBJECT(DrakonElement)

	PROCEDURE &Init*;
	BEGIN
		Init^;
		SetNameAsString(strDrakonElementNode)
	END Init;

	END DrakonElementNode;
	
VAR
	PrototypeIndex,
	PrototypeIndexL,
	PrototypeIndexR,
	PrototypeIndexT,
	PrototypeIndexB,
	PrototypeIndexX: WMProperties.Int32Property;
	
	PrototypeCode,
	PrototypeText: WMProperties.StringProperty;
	
	strDrakonDocument,
	strDrakonSchema,
	strDrakonElementHead,
	strDrakonElementAction,
	strDrakonElementEnd,
	strDrakonElementNode,
	strEnd: Strings.String;

PROCEDURE GenDrakonDocument*(): DrakonDocument;
VAR
	document: DrakonDocument;
BEGIN
	NEW(document); RETURN document
END GenDrakonDocument;

PROCEDURE GenDrakonSchema*(): DrakonSchema;
VAR
	schema: DrakonSchema;
BEGIN
	NEW(schema); RETURN schema
END GenDrakonSchema;

PROCEDURE GenDrakonElementHead*(): DrakonElementHead;
VAR
	element: DrakonElementHead;
BEGIN
	NEW(element); RETURN element
END GenDrakonElementHead;

PROCEDURE GenDrakonElementAction*(): DrakonElementAction;
VAR
	element: DrakonElementAction;
BEGIN
	NEW(element); RETURN element
END GenDrakonElementAction;

PROCEDURE GenDrakonElementEnd*(): DrakonElementEnd;
VAR
	element: DrakonElementEnd;
BEGIN
	NEW(element); RETURN element
END GenDrakonElementEnd;

PROCEDURE GenDrakonElementNode*(): DrakonElementNode;
VAR
	element: DrakonElementNode;
BEGIN
	NEW(element); RETURN element
END GenDrakonElementNode;

PROCEDURE InitPrototypes;
BEGIN
	NEW(PrototypeIndex, NIL, Strings.NewString("ElementIndex"), Strings.NewString("Index of the element"));
	NEW(PrototypeIndexL, NIL, Strings.NewString("ElementIndexLeft"), Strings.NewString("Index of the left element"));
	NEW(PrototypeIndexR, NIL, Strings.NewString("ElementIndexRight"), Strings.NewString("Index of the right element"));
	NEW(PrototypeIndexT, NIL, Strings.NewString("ElementIndexTop"), Strings.NewString("Index of the top element"));
	NEW(PrototypeIndexB, NIL, Strings.NewString("ElementIndexBottom"), Strings.NewString("Index of the bottom element"));
	NEW(PrototypeIndexX, NIL, Strings.NewString("ElementIndexExtra"), Strings.NewString("Index of the extra element"));
	NEW(PrototypeText, NIL, Strings.NewString("ElementText"), Strings.NewString("Text of the element"));
	NEW(PrototypeCode, NIL, Strings.NewString("ElementCode"), Strings.NewString("Code of the element"));
END InitPrototypes;


BEGIN
	strDrakonDocument := Strings.NewString("DrakonDocument");
	strDrakonSchema := Strings.NewString("DrakonSchema");
	strDrakonElementHead := Strings.NewString("DrakonElementHead");
	strDrakonElementAction := Strings.NewString("DrakonElementAction");
	strDrakonElementEnd := Strings.NewString("DrakonElementEnd");
	strDrakonElementNode := Strings.NewString("DrakonElementNode");
	strEnd := Strings.NewString("End");
	InitPrototypes;
END DrakonComponents.

SystemTools.Free DrakonComponents ~


DrakonComponents.CreateRepository ~

Repositories.Create Drakon.rep ~
Repositories.Load Drakon ~
Repositories.Put DrakonDocument Drakon DrakonDocument ~

Repositories.Load Standard ~
TestComponents.Test Button ~


Repositories.DumpAll ~