MODULE DrakonTest; (** AUTHOR "SAGE"; PURPOSE "DRAKON Visual Languge Test Procedures"; *)

IMPORT
	DrakonComponents, DrakonUtils, Commands, Files, Strings;
	
PROCEDURE TestCreate*(context: Commands.Context);
VAR
	document: DrakonComponents.DrakonDocument;
	schema: DrakonComponents.DrakonSchema;
	element1, element2: DrakonComponents.DrakonElement;
	filename: ARRAY 128 OF CHAR;
	res: LONGINT;
BEGIN

	context.arg.GetString(filename);
	
	IF Strings.Length(filename) = 0 THEN
		context.out.String("File name not specified!");
		context.out.Ln
	ELSE
		(* Create simple schema *)
		context.out.String("Creating schema...");
		context.out.Ln;	
		
		document := DrakonComponents.GenDrakonDocument();
		schema := DrakonComponents.GenDrakonSchema();
		
		document.AddContent(schema);
		
		context.out.String("Adding elements...");
		context.out.Ln;	
		
		element1 := DrakonComponents.GenDrakonElementHead();	schema.AddContent(element1);
		element1.text.Set(Strings.NewString("Calculate a + b")); element1.code.Set(Strings.NewString("PROCEDURE Calc(a, b: INTEGER): INTEGER;"));
					
		element2 := DrakonComponents.GenDrakonElementNode(); schema.AddContent(element2);
		
		element1.LinkBottomElement(element2);
		
		element1 := DrakonComponents.GenDrakonElementAction(); schema.AddContent(element1);
		element1.text.Set(Strings.NewString("a + b")); element1.code.Set(Strings.NewString("RETURN a + b"));
		
		element2.LinkBottomElement(element1);
		
		element2 := DrakonComponents.GenDrakonElementNode(); schema.AddContent(element2);
		
		element1.LinkBottomElement(element2);
		
		element1 := DrakonComponents.GenDrakonElementEnd(); schema.AddContent(element1);
		
		element2.LinkBottomElement(element1);
		
		context.out.String("Writing schema...");
		context.out.Ln;	

		DrakonUtils.Store(filename, document, res);

		IF res = DrakonComponents.Ok THEN
			context.out.String("Done!")
		ELSE
			context.out.String("Error! File not created!")
		END;
		context.out.Ln
	END;
END TestCreate;

PROCEDURE TestLoad*(context: Commands.Context);
VAR
	filename: ARRAY 128 OF CHAR;
	file: Files.File;
	writer: Files.Writer;
	document: DrakonComponents.DrakonDocument;
BEGIN

	context.arg.GetString(filename);
	
	IF Strings.Length(filename) = 0 THEN
		context.out.String("File name not specified!");
		context.out.Ln
	ELSE
		context.out.String("Trying to load document...");
		context.out.Ln;
		DrakonUtils.RegisterAndLoad(filename, document);
		IF document # NIL THEN
		
			context.out.String("Something loaded...");
			context.out.Ln;
			
			context.out.String("Something loaded...");
		
			file := Files.New("DrakonTest.XML");
			IF file # NIL THEN
				Files.OpenWriter(writer, file, 0);
				
				document.Write(writer, 0);
				
				writer.Update;
				Files.Register(file);
				
				context.out.String("Done!");
				context.out.Ln;	

			END
		END
	END	
END TestLoad;

END DrakonTest.

DrakonTest.TestCreate schema.XML ~
DrakonTest.TestLoad schema.XML ~

SystemTools.Free DrakonTest DrakonUtils DrakonComponents ~



------------------------------------
Repositories.Load Drakon ~
DrakonTest.TestLoad schema.XML ~
Repositories.Unload Drakon ~

TestComponents.Test DrakonDocument ~

