MODULE DrakonUtils; (** AUTHOR "SAGE"; PURPOSE "DRAKON Visual Languge Utilities"; *)

IMPORT
	DrakonComponents, XML, XMLScanner, XMLParser, Streams, Codecs, KernelLog, WMComponents, Files, Strings;

CONST
	Ok* = 0;
	Error* = -1;
	
VAR
	hasErrors: BOOLEAN;

PROCEDURE Load*(CONST filename: ARRAY OF CHAR; VAR document: DrakonComponents.DrakonDocument);
VAR
	content: XML.Content;
BEGIN
	document := NIL;
	content := WMComponents.Load(filename);
	IF content IS DrakonComponents.DrakonDocument THEN
		document := content(DrakonComponents.DrakonDocument)
	END
END Load;

(* Report errors while parsing *)
PROCEDURE LogError(pos, line, row: LONGINT; CONST msg: ARRAY OF CHAR);
BEGIN
	KernelLog.String("Parse error at pos "); KernelLog.Int(pos, 5); KernelLog.String(" in line "); KernelLog.Int(line, 5);
	KernelLog.String(" row "); KernelLog.Int(row, 5); KernelLog.String(" - "); KernelLog.String(msg); KernelLog.Ln;
	hasErrors := TRUE
END LogError;

(** Load an XML file. Return NIL if errors occured *)
PROCEDURE LoadWithRegistry(CONST filename : ARRAY OF CHAR; registry: XML.ElementRegistry): XML.Content;
VAR scanner : XMLScanner.Scanner;
	parser : XMLParser.Parser;
	doc : XML.Document;
	in : Streams.Reader;
BEGIN {EXCLUSIVE}
	in := Codecs.OpenInputStream(filename);
	IF in # NIL THEN
		NEW(scanner, in); scanner.reportError := LogError;
		NEW(parser, scanner); parser.reportError := LogError;
		parser.elemReg := registry; doc := parser.Parse();
		IF hasErrors THEN RETURN NIL END;
		RETURN doc.GetRoot()
	END;
	RETURN NIL
END LoadWithRegistry;

PROCEDURE RegisterAndLoad*(CONST filename: ARRAY OF CHAR; VAR document: DrakonComponents.DrakonDocument);
VAR
	content: XML.Content;
	registry: XML.ElementRegistry;
	name: Strings.String;
BEGIN

	(* Try to register elements' generators *)
	NEW(registry);
	registry.RegisterElementByName("DrakonDocument", "DrakonComponents", "GenDrakonDocument");
	registry.RegisterElementByName("DrakonSchema", "DrakonComponents", "GenDrakonSchema");
	registry.RegisterElementByName("DrakonElementHead", "DrakonComponents", "GenDrakonElementHead");
	registry.RegisterElementByName("DrakonElementAction", "DrakonComponents", "GenDrakonElementAction");
	registry.RegisterElementByName("DrakonElementEnd", "DrakonComponents", "GenDrakonElementEnd");
	registry.RegisterElementByName("DrakonElementNode", "DrakonComponents", "GenDrakonElementNode");
	
	document := NIL;
	content := LoadWithRegistry(filename, registry);
	
	IF content # NIL THEN
		KernelLog.String("xml loaded!");
		KernelLog.Ln
	END;
	
	IF content IS XML.Element THEN
		name := content(XML.Element).GetName();
		KernelLog.String(name^);
		KernelLog.Ln
	END;
	
	IF content IS DrakonComponents.DrakonDocument THEN
		KernelLog.String("content IS DrakonComponents.DrakonDocument");
		KernelLog.Ln;
		document := content(DrakonComponents.DrakonDocument)
	END
END RegisterAndLoad;

PROCEDURE Store*(CONST filename: ARRAY OF CHAR; document: DrakonComponents.DrakonDocument; VAR res: LONGINT);
VAR
	decl: XML.XMLDecl;
	file: Files.File;
	writer: Files.Writer;
BEGIN
	res := Error;
	file := Files.New(filename);
	IF file # NIL THEN
		Files.OpenWriter(writer, file, 0);

		NEW(decl);
		decl.SetStandalone(TRUE);
		decl.SetVersion("1.0");
		decl.SetEncoding("UTF-8");
		
		decl.Write(writer, 0);
		document.Write(writer, 0);
		
		writer.Update;
		Files.Register(file);
		
		res := Ok
	END
END Store;

(*
PROCEDURE CreateRepository*(context: Commands.Context);
VAR
	id, res: LONGINT;
BEGIN
	Repositories.CreateRepository("Drakon", res);
	IF res = Repositories.Ok THEN
		context.out.String("Repository created, puting components...");
		context.out.Ln;
		Repositories.PutComponent(DrakonComponents.GenDrakonDocument(), "Drakon", "DrakonComponents.GenDrakonDocument", id, res);
		Repositories.PutComponent(DrakonComponents.GenDrakonSchema(), "Drakon", "DrakonComponents.GenDrakonSchema", id, res)
	END;
END CreateRepository;
*)

END DrakonUtils.
